package cn.com.qmth.stmms.ms.collect.util;

import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.commons.utils.MD5Util;
import cn.com.qmth.stmms.ms.commons.utils.image.ImageCompression;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.LoggerFactory;
import org.springframework.util.FileCopyUtils;

import java.io.*;
import java.util.HashSet;
import java.util.Objects;

public class RotateTask implements Runnable {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(RotateTask.class);
    ImageConfig imageConfig;
    StudentRepo studentRepo;
    private Paper paper;
    private int degree;
    private PaperRepo paperRepo;
    private SystemConfig systemConfig;
    private HashSet<String> papers;

    public RotateTask(Paper paper, int degree, PaperRepo paperRepo, SystemConfig systemConfig, HashSet<String> papers, ImageConfig imageConfig, StudentRepo studentRepo) {
        this.paper = paper;
        this.degree = degree;
        this.paperRepo = paperRepo;
        this.systemConfig = systemConfig;
        this.papers = papers;
        this.imageConfig = imageConfig;
        this.studentRepo = studentRepo;
    }

    @Override
    public void run() {
        long start = System.currentTimeMillis();
        try {
            String thumbPath = null;
            String thumbNew = null;
            String imagePath = null;
            String imageNew = null;
            String sliceMD5 = null;
            LOGGER.info("准备生成缩略和裁切旋转图:{}", start);
            if (imageConfig.isImageEnc()) {
                File thumbTempFile = null;
                File imageTempFile = null;
                try {
                    Student student = studentRepo.findByWorkIdAndExamNumber(paper.getWorkId(), paper.getExamNumber());
                    String imageMd5 = MD5Util.getImageRuleMd5(paper.getWorkId(), paper.getSubject().ordinal(), paper.getAreaCode(), paper.getExamNumber(), student.getId());
                    thumbPath = systemConfig.getThumbDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                            + File.separator + paper.getAreaCode() + File.separator + imageMd5;
                    thumbNew = systemConfig.getThumbDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                            + File.separator + paper.getAreaCode() + File.separator + imageMd5;
                    imagePath = systemConfig.getImageDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                            + File.separator + paper.getAreaCode() + File.separator + imageMd5;
                    imageNew = systemConfig.getImageDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                            + File.separator + paper.getAreaCode() + File.separator + imageMd5;

                    //缩略图旋转生成
                    String thumbOldPath = thumbPath + ".jpg";
                    String thumbTempPath = thumbPath + "temp.jpg";
                    File thumbOldFile = new File(thumbOldPath);
                    //读取指定路径下面的文件
                    thumbTempFile = new File(thumbTempPath);
                    InputStream is = new FileInputStream(thumbOldPath);
                    OutputStream outputStream = new FileOutputStream(thumbTempPath);
                    SystemConstant.writeStream(is, outputStream);
                    is.close();
                    outputStream.flush();
                    outputStream.close();

                    //读取原图
                    File thumbNewFile = new File(thumbNew + "_" + start + ".jpg");
                    FileCopyUtils.copy(thumbTempFile, thumbNewFile);
                    ImageCompression.rotate(thumbTempFile, thumbTempFile, degree);
                    FileCopyUtils.copy(thumbTempFile, thumbOldFile);
                    InputStream is1 = new ByteArrayInputStream(FileCopyUtils.copyToByteArray(thumbNewFile));
                    //加密原图
                    OutputStream outputStream1 = new FileOutputStream(thumbNewFile);
                    SystemConstant.writeStream(is1, outputStream1);
                    is1.close();
                    outputStream1.flush();
                    outputStream1.close();

                    //生成旋转图和加密
                    InputStream is2 = new ByteArrayInputStream(FileCopyUtils.copyToByteArray(thumbOldFile));
                    OutputStream outputStream2 = new FileOutputStream(thumbOldFile);
                    SystemConstant.writeStream(is2, outputStream2);
                    is2.close();
                    outputStream2.flush();
                    outputStream2.close();

                    //裁切原图旋转生成
                    String imageOldPath = imagePath + ".jpg";
                    String imageTempPath = imagePath + "temp.jpg";
                    File imageOldFile = new File(imageOldPath);
                    //读取指定路径下面的文件
                    imageTempFile = new File(imageTempPath);
                    InputStream is3 = new FileInputStream(imageOldPath);
                    OutputStream outputStream3 = new FileOutputStream(imageTempFile);
                    SystemConstant.writeStream(is3, outputStream3);
                    is3.close();
                    outputStream3.flush();
                    outputStream3.close();

                    //读取原图
                    File imageNewFile = new File(imageNew + "_" + start + ".jpg");
                    FileCopyUtils.copy(imageTempFile, imageNewFile);
                    ImageCompression.rotate(imageTempFile, imageTempFile, degree);
                    FileCopyUtils.copy(imageTempFile, imageOldFile);
                    InputStream is4 = new ByteArrayInputStream(FileCopyUtils.copyToByteArray(imageNewFile));
                    //加密原图
                    OutputStream outputStream4 = new FileOutputStream(imageNewFile);
                    SystemConstant.writeStream(is4, outputStream4);
                    is4.close();
                    outputStream4.flush();
                    outputStream4.close();

                    //生成旋转图和加密
                    InputStream is5 = new ByteArrayInputStream(FileCopyUtils.copyToByteArray(imageOldFile));
                    OutputStream outputStream5 = new FileOutputStream(imageOldFile);
                    SystemConstant.writeStream(is5, outputStream5);
                    is5.close();
                    outputStream5.flush();
                    outputStream5.close();

                    InputStream is6 = new FileInputStream(imageNewFile);
                    sliceMD5 = DigestUtils.md5Hex(is6);
                    is6.close();
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
//                    System.gc();
                    if (Objects.nonNull(thumbTempFile) && thumbTempFile.exists()) {
                        thumbTempFile.delete();
                    }
                    if (Objects.nonNull(imageTempFile) && imageTempFile.exists()) {
                        imageTempFile.delete();
                    }
                }
            } else {
                thumbPath = systemConfig.getThumbDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                        + File.separator + paper.getAreaCode() + File.separator + paper.getExamNumber() + ".jpg";
                thumbNew = systemConfig.getThumbDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                        + File.separator + paper.getAreaCode() + File.separator + paper.getExamNumber() + "_" + start + ".jpg";
                imagePath = systemConfig.getImageDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                        + File.separator + paper.getAreaCode() + File.separator + paper.getExamNumber() + ".jpg";
                imageNew = systemConfig.getImageDir() + File.separator + paper.getWorkId() + File.separator + paper.getSubject().toString()
                        + File.separator + paper.getAreaCode() + File.separator + paper.getExamNumber() + "_" + start + ".jpg";
                File thumbFile = new File(thumbPath);
                if (thumbFile.exists()) {
                    thumbFile.renameTo(new File(thumbNew));
                    InputStream is = new FileInputStream(thumbNew);
                    ImageCompression.rotate(new File(thumbNew), new File(thumbPath), degree);
                    is.close();
                }

                File imageFile = new File(imagePath);
                if (imageFile.exists()) {
                    imageFile.renameTo(new File(imageNew));
                    InputStream is = new FileInputStream(imageNew);
                    ImageCompression.rotate(new File(imageNew), new File(imagePath), degree);
                    is.close();
                }
                FileInputStream in = new FileInputStream(new File(imagePath));
                sliceMD5 = DigestUtils.md5Hex(in);
                in.close();
            }
            long end = System.currentTimeMillis();
            LOGGER.info("生成缩略和裁切旋转图耗时：{},sliceMD5:{}", (end - start) / 1000 + "s", sliceMD5);
            paper.setSliceMD5(sliceMD5);
            paperRepo.save(paper);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            papers.remove(paper.getExamNumber());
        }
    }
}
